@@ -4,10 +4,10 @@ use 5.006;
 
 use strict;
 
-our $VERSION = '0.95_02';
+our $VERSION = '0.98_01';
 $VERSION = eval $VERSION;    ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
-use Test::Builder::Module;
+use Test::Builder::Module 0.98;
 our @ISA    = qw(Test::Builder::Module);
 our @EXPORT = qw(ok);
 
